
-- postgresql doenst support this correctly, refers to check variable as value instead of by domain name.
-- create domain string as varchar check (string <> '');

create table accounts
(
username        varchar(30)     NOT NULL CHECK (username <> ''),
pass            char(64)        NOT NULL CHECK (pass <> ''),
type            varchar(30)     NOT NULL CHECK (type <> ''),
email           varchar(30)     NOT NULL CHECK (email <> ''),
PRIMARY KEY (username)
);

create table students
(
username        varchar(30)     NOT NULL CHECK (username <> ''),
id              int             NOT NULL CHECK (id > 0),
name            varchar(90)     NOT NULL CHECK (name <> ''),
major           varchar(30)     default 'Undeclared',
UNIQUE(id),
PRIMARY KEY (username),
FOREIGN KEY (username) references accounts(username) ON UPDATE CASCADE ON DELETE CASCADE
);

create table courses
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
title           varchar(100)    NOT NULL,
credits         int             CHECK (credits > 0 and credits <= 10) default '3',
description     varchar(255),
PRIMARY KEY (course)
);

create table sections
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
section         int             NOT NULL CHECK (section > 0),
semester        varchar(10)     NOT NULL CHECK (semester <> ''),
year            int             NOT NULL CHECK (year > 2000 and year < 3000),
instructor      varchar(60)     default 'TBA',
schedule        varchar(100)    default 'TBA',
location        varchar(30)     default 'TBA',
PRIMARY KEY (course, section, semester, year),
FOREIGN KEY (course) references courses(course) ON UPDATE CASCADE ON DELETE CASCADE
);

create table enrollment
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
section         int             NOT NULL CHECK (section > 0),
semester        varchar(10)     NOT NULL CHECK (semester <> ''),
year            int             NOT NULL CHECK (year > 2000 and year < 3000),
id              int             NOT NULL CHECK (id > 0),
PRIMARY KEY (course, section, semester, year, id),
FOREIGN KEY (course, section, semester, year) references sections(course, section, semester, year) ON UPDATE CASCADE ON DELETE CASCADE,
FOREIGN KEY (id) references students(id) ON UPDATE CASCADE ON DELETE CASCADE
);

create table assignments
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
section         int             NOT NULL CHECK (section > 0),
semester        varchar(10)     NOT NULL CHECK (semester <> ''),
year            int             NOT NULL CHECK (year > 2000 and year < 3000),
assignment      varchar(100)    NOT NULL CHECK (assignment <> ''),
due             DATE,
percentage      int		NOT NULL CHECK ( percentage >= 0 and percentage <= 100 ),
description     varchar(255),
PRIMARY KEY (course, section, semester, year, assignment),
FOREIGN KEY (course, section, semester, year) references sections(course, section, semester, year) ON UPDATE CASCADE ON DELETE CASCADE
);


-- CREATE ASSERTION percentage_constraint
-- CHECK
-- (
-- 	EXISTS
-- 	(
-- 		SELECT course, section, semester, year, SUM(percentage)
-- 		FROM assignments
-- 		GROUP BY course, section, semester, year
-- 		HAVING SUM(percentage) > 100
-- 	)
-- );

create table questions
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
section         int             NOT NULL CHECK (section > 0),
semester        varchar(10)     NOT NULL CHECK (semester <> ''),
year            int             NOT NULL CHECK (year > 2000 and year < 3000),
assignment      varchar(100)    NOT NULL CHECK (assignment <> ''),
question        int,
nCorrect        int,
nAttempted      int,
PRIMARY KEY (course, section, semester, year, assignment),
FOREIGN KEY (course, section, semester, year, assignment) references assignments(course, section, semester, year, assignment) ON UPDATE CASCADE ON DELETE CASCADE
);

create table grades
(
course          varchar(100)    NOT NULL CHECK (course <> ''),
section         int             NOT NULL CHECK (section > 0),
semester        varchar(10)     NOT NULL CHECK (semester <> ''),
year            int             NOT NULL CHECK (year > 2000 and year < 3000),
assignment      varchar(100)    NOT NULL CHECK (assignment <> ''),
id              int             NOT NULL CHECK (id > 0),
grade           int             NOT NULL CHECK (grade >= 0 and grade <= 100),
PRIMARY KEY (course, section, semester, year, assignment, id),
FOREIGN KEY (course, section, semester, year, assignment) references assignments(course, section, semester, year, assignment) ON UPDATE CASCADE ON DELETE CASCADE
);